<?php
/**
 * @package     CSVI
 * @subpackage  RsForm
 *
 * @author      Roland Dalmulder <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        http://www.csvimproved.com
 */

defined('_JEXEC') or die;

require_once JPATH_ADMINISTRATOR . '/components/com_csvi/models/exports.php';

/**
 * Export RSForm! Submissions.
 *
 * @package     CSVI
 * @subpackage  RsForm
 * @since       6.6.0
 */
class Com_RsformModelExportSubmission extends CsviModelExports
{
	/**
	 * Array of cached form names
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $formTitles = array();

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 *
	 * @throws  CsviException
	 * @throws  RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Load the core fields
			$coreFields = $this->loadCoreFields();

			// Build something fancy to only get the fieldnames the user wants
			$userFields   = array();
			$whereFields  = array();
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby       = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// We must sort by submission ID to keep the records together
			array_unshift($sortbyfields, $this->db->quoteName('sv.SubmissionId'));

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'FormTitle':
						$userfields[] = $this->db->quoteName('FormId');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('FormId');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('FormId');
						}
						break;
					case 'custom':
						break;
					default:
						// Do not include custom fields into the query
						if (in_array($field->field_name, $coreFields))
						{
							$userFields[] = $this->db->quoteName($field->field_name);

							if (array_key_exists($field->field_name, $groupbyfields))
							{
								$groupby[] = $this->db->quoteName($field->field_name);
							}

							if (array_key_exists($field->field_name, $sortbyfields))
							{
								$sortby[] = $this->db->quoteName($field->field_name);
							}
						}
						else
						{
							$whereFields[] = $this->db->quote($field->field_name);
						}
						break;
				}
			}

			// Build the query
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $coreFields));
			$query->from($this->db->quoteName('#__rsform_submissions', 's'));

			// Filter by language
			$language = $this->template->get('language');

			if ($language && $language !== '*')
			{
				$query->where($this->db->quoteName('s.Lang') . ' = ' . $this->db->quote($language));
			}

			// Filter by form
			$forms = $this->template->get('forms');

			if ($forms && $forms[0] != '*')
			{
				$query->where($this->db->quoteName('s.FormId') . " IN ('" . implode("','", $forms) . "')");
			}

			// Filter by dates
			// Check for a pre-defined date
			$daterange = $this->template->get('daterange', '');

			if ($daterange != '')
			{
				$jdate       = JFactory::getDate();
				$currentDate = $this->db->quote($jdate->format('Y-m-d'));

				switch ($daterange)
				{
					case 'lastrun':
						if (strpos($this->template->getLastrun(), '0000') !== 0)
						{
							$query->where($this->db->quoteName('s.DateSubmitted') . ' > ' . $this->db->quote($this->template->getLastrun()));
						}
						break;
					case 'yesterday':
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') = DATE_SUB(' . $currentDate . ', INTERVAL 1 DAY)');
						break;
					case 'thisweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek - 1;
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= DATE_SUB(' . $currentDate . ', INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') <= ' . $currentDate);
						break;
					case 'lastweek':
						// Get the current day of the week
						$dayofweek = $jdate->__get('dayofweek');
						$offset = $dayofweek + 6;
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= DATE_SUB(' . $currentDate . ', INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') <= DATE_SUB(' . $currentDate . ', INTERVAL ' . $dayofweek . ' DAY)');
						break;
					case 'thismonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= DATE_SUB(' . $currentDate . ', INTERVAL ' . $dayofmonth . ' DAY)');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') <= ' . $currentDate);
						break;
					case 'lastmonth':
						// Get the current day of the week
						$dayofmonth = $jdate->__get('day');
						$month = date('n');
						$year = date('y');

						if ($month > 1)
						{
							$month--;
						}
						else
						{
							$month = 12;
							$year--;
						}

						$daysinmonth = date('t', mktime(0, 0, 0, $month, 25, $year));
						$offset = ($daysinmonth + $dayofmonth) - 1;

						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= DATE_SUB(' . $currentDate . ', INTERVAL ' . $offset . ' DAY)');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') <= DATE_SUB(' . $currentDate . ', INTERVAL ' . $dayofmonth . ' DAY)');
						break;
					case 'thisquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						switch ($quarter)
						{
							case '1':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-01-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-04-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-07-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-10-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year++ . '-01-01'));
								break;
						}
						break;
					case 'lastquarter':
						// Find out which quarter we are in
						$month = $jdate->__get('month');
						$year = date('Y');
						$quarter = ceil($month / 3);

						if ($quarter == 1)
						{
							$quarter = 4;
							$year--;
						}
						else
						{
							$quarter--;
						}

						switch ($quarter)
						{
							case '1':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-01-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-04-01'));
								break;
							case '2':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-04-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-07-01'));
								break;
							case '3':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-07-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-10-01'));
								break;
							case '4':
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-10-01'));
								$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year++ . '-01-01'));
								break;
						}
						break;
					case 'thisyear':
						$year = date('Y');
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-01-01'));
						break;
					case 'lastyear':
						$year = date('Y');
						$year--;
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') >= ' . $this->db->quote($year . '-01-01'));
						$year++;
						$query->where('DATE(' . $this->db->quoteName('s.DateSubmitted') . ') < ' . $this->db->quote($year . '-01-01'));
						break;
				}
			}
			else
			{
				// Filter by order date start
				$datestart = $this->template->get('datestart', false);

				if ($datestart)
				{
					$orderdate = JFactory::getDate($datestart);
					$query->where($this->db->quoteName('s.DateSubmitted') . ' >= ' . $this->db->quote($orderdate->toSql()));
				}

				// Filter by order date end
				$dateend = $this->template->get('dateend', false);

				if ($dateend)
				{
					$orderdate = JFactory::getDate($dateend);
					$query->where($this->db->quoteName('s.DateSubmitted') . ' <= ' . $this->db->quote($orderdate->toSql()));
				}
			}

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Load the details per submission
					$userFields = array_unique($userFields);

					$query->clear()
						->select($this->db->quoteName('FieldName'))
						->select($this->db->quoteName('FieldValue'))
						->from($this->db->quoteName('#__rsform_submission_values'))
						->where($this->db->quoteName('FieldName') . ' IN (' . implode(',', $whereFields) . ')')
						->where($this->db->quoteName('SubmissionId') . ' = ' . (int)  $record->SubmissionId);
					$this->db->setQuery($query);

					$this->log->add('Looking for form values');

					$data = $this->db->loadObjectList('FieldName');

					foreach ($exportfields as $field)
					{
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'FormTitle':
								if (!in_array($record->FormId, $this->formTitles))
								{
									$query->clear()
										->select($this->db->quoteName('FormTitle'))
										->from($this->db->quoteName('#__rsform_forms'))
										->where($this->db->quoteName('FormId') . ' = ' . (int) $record->FormId);
									$this->db->setQuery($query);

									$this->formTitles[$record->FormId] = $this->db->loadResult();
								}

								$fieldvalue = $this->formTitles[$record->FormId];
								break;
							case 'DateSubmitted':
								$date = JFactory::getDate($record->$fieldname);
								$fieldvalue = date($this->template->get('export_date_format'), $date->toUnix());
								break;
							default:
								foreach ($data as $dataFieldName => $dataFieldValue)
								{
									if ($dataFieldName == $fieldname)
									{
										$fieldvalue = $dataFieldValue->FieldValue;
										break;
									}
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}

	/**
	 * Load the RSForm! core fields.
	 *
	 * @return  array  List of core fields.
	 *
	 * @since   6.6.0
	 */
	private function loadCoreFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('csvi_name'))
			->from($this->db->quoteName('#__csvi_availablefields'))
			->where($this->db->quoteName('component_table') . ' = ' . $this->db->quote('rsform_submissions'))
			->where($this->db->quoteName('action') . ' = ' . $this->db->quote('export'));
		$this->db->setQuery($query);

		return $this->db->loadColumn();
	}
}
